/**
 *  Source code for the SimpleSimulation class
 *  Copyright (C) 2014  Jamie Fairbrother
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <iomanip>
#include <stdexcept>
#include <vector>
#include <cmath>
#include <ctime>

#include <gsl/gsl_cdf.h>

#include "SimpleSimulation.hpp"

static const bool dbg = false;

SimpleSimulation::SimpleSimulation(std::function<double(std::mt19937&)> dist,
				   bool verbose)
  : SimulationBase(dist, verbose)
{
  if (dbg) std::cout << "SimpleSimulation: construction successful." << std::endl;
}

SimpleSimulation::SimpleSimulation(const SimpleSimulation& sim)
  : SimulationBase(sim.dist, sim.verbose)
{
  if (dbg) std::cout << "Copy construction successful." << std::endl;
}


void SimpleSimulation::accumulate() {
  std::mt19937 rng;
  double old_mean, om2;
  double x;

  while (true)
    {
      // Update stuff
      ++n;
      old_mean = mean;

      // Simulate
      x = dist(rng);
      
      // Update means and sum_squares
      mean += (x - old_mean)/(double)n;
      om2 = old_mean*old_mean;
      var += om2-mean*mean+(x*x - var - om2)/(double)n;

      // Print message and stop check
      if (n % batch_size == 0)
	{
	  if (verbose) message();
	  if (check_complete()) break;
	}
    }
}
