/**
 *  Header file for SimpleSimulation class
 *  Copyright (C) 2014  Jamie Fairbrother
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef SIMPLE_SIMULATION_HPP
#define SIMPLE_SIMULATION_HPP

#include <functional>
#include <random>
#include "SimulationBase.hpp"

/**
 * Class for running a simple serial simulation
 */

class SimpleSimulation : public SimulationBase
{
public:

  /**
   * \brief Construct instance of simulation.
   * \param rng Random number generator.
   * \param verbose Whether simulation information should be printed to standard output.
   */
  SimpleSimulation(std::function< double(std::mt19937 &) > dist,
		   bool verbose = false);

  virtual ~SimpleSimulation() {}

  SimpleSimulation(const SimpleSimulation&);

private:
  SimpleSimulation();           // Forbid default constructor
  virtual void accumulate();
};

#endif
