/**
 *  Example program which simulates the mean
 *  of a uniform random variable using both simulation classes.
 *  Copyright (C) 2014  Jamie Fairbrother
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


#include <cstdlib>
#include <random>
#include <iostream>
#include "SimpleSimulation.hpp"
#include "ParallelSimulation.hpp"

using namespace std;

int main()
{
  // Simulation running parameters
  //std::cout << (double)1/2 << std::endl;
  int batch_size = 10000000;
  double eps_abs = 0.0005;
  double eps_rel = 0.00005;

  // Create a distribution for 
  uniform_real_distribution<double> unif;
  auto dist = [&](std::mt19937 &gen){return 100*unif(gen) + 5.0;};

  // Create and run serial simulation
  cout << "Running serial simulation..." << endl;
  SimpleSimulation sim1(dist,true);
  sim1.set_eps_abs(eps_abs).set_eps_rel(eps_rel).set_batch_size(batch_size);
  sim1.run();

  // Create and run parallel simulation
  cout << "\nRunning parallel simulation..." << endl;
  ParallelSimulation sim2(dist, 4, true);
  sim2.set_eps_abs(eps_abs).set_eps_rel(eps_rel).set_batch_size(batch_size);
  sim2.run();
}
