/**
 *  Checks the correlation between pseudo-random number generators 
 *  with different seeds. We would like this to be zero.
 *  Copyright (C) 2014  Jamie Fairbrother
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <vector>
#include <iomanip>
#include <random>

using namespace std;

int main() {
  const size_t num_gens = 5;
  const int n = 10000000;
  double means[num_gens];
  double covs[num_gens][num_gens];
  static double samples[n][num_gens];
  vector<mt19937> gens(num_gens);
  uniform_real_distribution<double> distribution(0.0,1.0);

  for (size_t j = 0; j < num_gens; ++j) gens[j].seed(100*j+1);
  
  for (int i = 0; i < n; ++i) {
    for (size_t j = 0; j < num_gens; ++j) 
      samples[i][j] = distribution(gens[j]);
  }

  // for (int k = 0; k < n; ++k) {
  //   for (size_t j = 0; j < num_gens; ++j) 
  //     cout << setw(7) << setprecision(3) << samples[k][j];
  //   cout << endl;
  // }

  // Calculate means
  for (size_t i = 0; i < num_gens; ++i) means[i] = 0.0;
  for (size_t i = 0; i < num_gens; ++i) {
    for (int k = 0; k < n; ++k) means[i] += samples[k][i];
    means[i] /= n;
  }

  // Calculate covariance
  for (size_t i = 0; i < num_gens; ++i)
    for (size_t j = 0; j < num_gens; ++j) covs[i][j] = 0.0;
  for (size_t i = 0; i < num_gens; ++i) {
    for (size_t j = 0; j <= i; ++j) {
      for (int k = 0; k < n; ++k) covs[i][j] += ((samples[k][i] - means[i]) * (samples[k][j] - means[j]));
      covs[i][j] /= (n - 1);
      covs[j][i] = covs[i][j];
    }
  }

  // Print results
  cout << "Means: " << endl;
  for (size_t i = 0; i < num_gens; ++i) cout << fixed << setw(6) << setprecision(3) << means[i];
  cout << "\nCovariance: " << endl;
  for (size_t i = 0; i < num_gens; ++i) {
    for (size_t j = 0; j <= i; ++j) cout << fixed << setw(6) << setprecision(2) << covs[i][j];
    cout << endl;
  }

    
}
